/************************************************
*                                               *
*        Copyright (c) Alexandr Oshurkov        *
*                                               *
*    http://www.softonic.narod.ru/index.html    *
*                                               *
*              softonic@narod.ru                *
*                                               *
*************************************************/

#if !defined(AFX_AUTOPTR_H__4996_E57F_4ABB_B4BD_6784244D1BB8__INCLUDED_)
#define AFX_AUTOPTR_H__4996_E57F_4ABB_B4BD_6784244D1BB8__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <windows.h>
#include <memory.h>

template<class Type>
class CAutoWinHeap
{
	PVOID   m_pMemory;
	DWORD   m_size;
	HANDLE  m_hHeap;

public:
	CAutoWinHeap() : m_pMemory(0) , m_size(0), m_hHeap(0)
	{
		m_hHeap = ::GetProcessHeap();
	};

	CAutoWinHeap(DWORD dwSize) : m_pMemory(0) , m_hHeap(0)
	{
		m_hHeap = ::GetProcessHeap();
		AllocBytes(dwSize);
	};

	CAutoWinHeap( CAutoWinHeap<Type>& theHeap) : m_pMemory(0) , m_size(0), m_hHeap(0)
	{
		m_hHeap = ::GetProcessHeap();
		if (theHeap.IsValid())
			if (AllocBytes(theHeap.m_size))
				::CopyMemory(m_pMemory , theHeap.m_pMemory , m_size);
	} 

	virtual ~CAutoWinHeap()
	{
		Free();
	};

	DWORD GetSize() const
	{
		return m_size;
	}

	CAutoWinHeap<Type>& operator =(const CAutoWinHeap<Type>& theHeap)
	{
		if (this == &theHeap)
			return *this;
		if (theHeap.IsValid())
			if (AllocBytes(theHeap.m_size))
				::CopyMemory(m_pMemory , theHeap.m_pMemory , m_size);
		return *this;
	}

	operator Type*() const
	{
		return static_cast<Type*> (m_pMemory);
	}

	operator bool() const
	{
		return IsValid();
	}

	bool IsValid() const
	{
		return m_pMemory != NULL;
	}
	
	bool AllocBytes(DWORD size)
	{
		if (size == 0)
			return false;

		if (m_pMemory != NULL)
			m_pMemory = ::HeapReAlloc(m_hHeap , HEAP_ZERO_MEMORY , m_pMemory , size);
		else
			m_pMemory = ::HeapAlloc(m_hHeap , HEAP_ZERO_MEMORY , size);

		if (m_pMemory != NULL)
		{
			m_size = size;
			return true;
		}
		return false;
	}

	bool AllocObjects(DWORD num = 1)
	{
		if (num == 0)
			return false;

		DWORD sizeob = sizeof(Type);

		if (m_pMemory != NULL)
			m_pMemory = ::HeapReAlloc(m_hHeap , HEAP_ZERO_MEMORY , m_pMemory , num*sizeob);
		else
			m_pMemory = ::HeapAlloc(m_hHeap , HEAP_ZERO_MEMORY , num*sizeob);

		if (m_pMemory != NULL)
		{
			m_size = num*sizeob;
			return true;
		}
		return false;
	}

	void Free()
	{
		if (m_pMemory != NULL)
		{
			::HeapFree(m_hHeap , 0 , m_pMemory);
			m_pMemory = 0;
			m_size    = 0;
		}
	}

	void ResetMemory()
	{
		if (m_pMemory)
			::ZeroMemory(m_pMemory , m_size);
	}
};


#endif // !defined(AFX_AUTOPTR_H__4996_E57F_4ABB_B4BD_6784244D1BB8__INCLUDED_)
